#
# ZBabelTag
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472      ABN: 83 082 081 472
# All Rights Reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#  1. Redistributions of source code must retain the above copyright
#         notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#         notice, this list of conditions and the following disclaimer in the
#         documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
#
# Author: Andrew Milton <akm@theinternet.com.au>
#
# USAGE:
#
# <dtml-babel src="language" dst="language">
# TheText
# </dtml-babel>
#
# src: is the language that TheText written in
# dst: is the desired language that TheText is to be displayed in.
#
# You can use any designations you like for language.

from Globals                  import HTMLFile
from AccessControl.Role       import RoleManager
from OFS.SimpleItem           import SimpleItem
from Products.ZSQLMethods.SQL import SQL
from SQLContainerSupport      import SQLContainerSupport
from LabelSupport             import LabelSupport
from ImportExportSupport      import ImportExportSupport

import md5, base64
import ZBabelSQL
import string


manage_addZBabelTowerForm = HTMLFile('manageAdd',globals())

def manage_addZBabelTower(self, id, connection, title = None, REQUEST=None):
    """manage_addZBabelTower(self, id, connection, title = None, REQUEST=None) --> add a Babel Tower
       to the ZODB; return None or HTML"""

    ob = ZBabelTower(id, connection)
    self._setObject(id, ob)
    
    if REQUEST:
        return self.manage_main(self,REQUEST)



class ZBabelTower(SimpleItem, RoleManager, SQLContainerSupport, LabelSupport, ImportExportSupport):
    ''' ZBabelTower - A class that implementd the dictionary of the ZBabel Translation System '''

    # The official Zope name for the class
    meta_type='ZBabel Tower'

    # Define the tabs, that will be visible from the Front End
    manage_options = ( ({'label': 'Translate',   'action': 'manage_Translate'},) +
                       LabelSupport.manage_options +  
                       SQLContainerSupport.manage_options +  
                       ImportExportSupport.manage_options +
                       ({'label': 'Security',    'action': 'manage_access'},
                        {'label': 'Undo',        'action': 'manage_UndoForm'},) )

    # Define the security for this object.
    __ac_permission__ = (('Manage ZBabel Towers', ('manage_main', 'manage',
                                                   'manage_workspace',
                                                   'manage_Translate',
                                                   'reCatalog')),)

    # Define the SQL MEthods that are used by the object
    sqlMethods = ( (('sql_initPhrases',        'Initialize the Phrases Table', 'dropFirst:int:required'),
                    ('sql_newPhrase',          'Insert a New Phrase', 'lang:required md5Hash:required '+
                     'renderPath:required renderText:required'),
                    ('sql_updatePhrase',       'Update a Single Phrase', 'phrase_id:int:required renderText:required'),
                    ('qry_findMatches',        'Find Matching Phrases', 'md5Hash:required'),
                    ('qry_getAllPhrases',      'Get All Phrases', ''),
                    ('qry_getPhrasesByPage',   'Get Phrases For on Page', 'renderPath srcLang dstLang'),
                    ('qry_getLanguages',       'Get Existing Languages', ''),
                    ('qry_getmd5Hashes',       'Get list of Unique md5Hashes', 'srcLang'),
                    ('qry_getPages',           'Get Pages with Translations', ''),
                    ('qry_getPhrasesToUpdate', 'Force Translation Table Fill', 'srcLang:required dstLang:required'))
                    + LabelSupport.sqlMethods )
    
    
    manage_main       = manage_Translate = HTMLFile('Translations', globals())
    translatePage     = HTMLFile('translatePage', globals())
    
    
    def __init__(self, id, connection, title=None):
        '''__init__(self, id, connection, title=None) --> Inialize the object; return None'''
        self.connection = connection
        self.id         = id
        self.title      = title
        self.registerSQLQueries()


    def registerSQLQueries(self):
        '''registerSQLQueries(self) --> iniitialize all the SQL Methods; return None'''
        for method in self.sqlMethods:
            sql = getattr(ZBabelSQL, '_' + method[0])
            object = SQL(method[0], method[1], self.connection, method[2], sql)
            setattr(self, method[0], object)
            
        self.qry_findMatches.manage_advanced(max_rows=5000, max_cache=5000,
                                            cache_time=3600, class_name='', class_file='')


    def findTranslation(self, digest, srcText, src, dst, srcPath, makeNewEntries=1):
        '''findTranslation(self, digest, srcText, src, dst, srcPath) --> get the translation of the
           srcText; return string'''

        # try to get the srcText in all langauges from the DB
        matches = self.qry_findMatches(md5Hash = digest)
        
        if matches:
            for hit in matches:
                if sqlattr(hit, 'lang') == dst:
                    return sqlattr(hit,'rendertext')
        
            # Insert a new phrase to be translated in
            # The destination language...
            if makeNewEntries:
                self.sql_newPhrase(lang = dst, md5Hash = digest, renderPath = srcPath,
                                   renderText = srcText)
        
            # Return our native text
            return srcText
        
        # This phrase is not registered at all
        # Register it...
        if makeNewEntries:
            self.sql_newPhrase(lang = src, md5Hash = digest, renderPath = srcPath,
                               renderText = srcText)
        
        # It's not registered and we also wanted a translation
        # Enter the destination translation
        if src != dst and makeNewEntries:
            self.sql_newPhrase(lang=dst, md5Hash=digest, renderPath=srcPath,
                               renderText=srcText)
            
        return srcText


    def _makeDigest(self, srcText):
        '''_makeDigest(self, srcText) -->  make a MD5 Hash Digest from the source text;
           return string'''
        digest = md5.new()
        digest.update(srcText)
        digest = digest.digest()
        digest = string.strip(base64.encodestring(digest))
        return digest

            
    def translate(self, srcText, src, dst, srcPath, makeNewEntries=1):
        '''translate(self, srcText, src, dst, srcPath) --> translate the srcText to language dst;
           return string'''

        digest = self._makeDigest(srcText)
        matchText = self.findTranslation(digest, srcText, src, dst, srcPath, makeNewEntries)

        return matchText
    
            
    def updateTranslationDB(self, srcLang, dstLang, REQUEST = None):
        '''updateTranslationDB(self, srcLang, dstLang, REQUEST = None) --> Force translation
           entries into a database; return None or HTML'''
    
        if srcLang == dstLang:
            if REQUEST:
                return self.manage_Translate(self,REQUEST)
            return None
            
        phrases = self.qry_getPhrasesToUpdate(srcLang = srcLang, dstLang = dstLang)
        if phrases:
            for phrase in phrases:
                self.sql_newPhrase(lang       = dstLang,
                                   md5Hash    = sqlattr(phrase,'md5hash'),
                                   renderPath = sqlattr(phrase,'renderpath'),
                                   renderText = sqlattr(phrase,'rendertext'))
    
        if REQUEST:
            return self.manage_Translate(self, REQUEST)
    
    
    def updatePhrases(self,REQUEST=None):
        """ Update all Phrases on a Page """
        if REQUEST:
            for formVar in REQUEST.form.keys():
                if formVar[:7] == 'Phrase-':
                    crap, phraseId, lang = string.split(formVar,'-')
                    text = REQUEST.form[formVar]
                    self.sql_updatePhrase(phrase_id = int(phraseId), lang = lang,
                                          renderText = string.strip(text))
    
            return self.manage_Translate(self,REQUEST)
        


# Whee, this should be probably in a differnt module file
from string import upper, lower
import Missing
mt = type(Missing.Value)

def typeconv(val):
        if type(val)==mt:
                return ''
        return val

def sqlattr(ob, attr):
        name=attr
        if hasattr(ob, attr):
                return typeconv(getattr(ob, attr))
        attr=upper(attr)
        if hasattr(ob, attr):
                return typeconv(getattr(ob, attr))
        attr=lower(attr)
        if hasattr(ob, attr):
                return typeconv(getattr(ob, attr))
        raise NameError, name

